from __future__ import nested_scopes

import sys, string

import packagelist, config

__version__ = "$Revision: 1.8 $"

def resolveDependencies(packageName):
    """Get a list of all packages that need to be installed for packageName"""

    foo = 1
    installpkgs = config.InstalledPackages()
    pkgs = packagelist.getPackageList()
    depends = pkgs.getDepends(packageName)
    pkginfo = {}
    for package in depends.keys():
        if not installpkgs.installed(package):
            temp = resolveDependencies(package)
            for key in temp.keys():
                if not pkginfo.has_key(key):
                    pkginfo[key] = { 'size':long(pkgs.getSize(key))}
            if not pkginfo.has_key(package):
                pkginfo[package] = { 'size':long(pkgs.getSize(package))}
    return pkginfo

def listAvailable(input):
    """List available packages and their versions and sizes"""

    packages = packagelist.getPackageList()
    available = packages.listAll()
    i = 0
    sys.stdout.write("%s  %s %s\n" % (string.ljust('Name', 30),
                                      string.ljust('Version', 20),
                                      string.ljust('Size', 10)))
    for (name, version, size) in available:
        i += 1
        sys.stdout.write("%s  %10s %10s\n" % (string.ljust(name, 30),
                                              string.ljust(version, 20),
                                              string.ljust(str(size), 10)))
    sys.stdout.write("%d total packages\n" % i)

def getDependency(input):
    """Find and print dependency information for a package"""
    if len(input) != 2:
        sys.stderr.write("Invalid usuage - please see help")
    else:
        packageName = input[1]
        try:
            depends = resolveDependencies(packageName)
        except packagelist.NotFound:
            sys.stdout.write("Package not found in database\n")
            return

        size = 0L
        sys.stdout.write("Dependencies: \n")
        for key in depends.keys():
            temp = depends[key]
            sys.stdout.write("%s -  %d bytes\n" % (key, temp['size']))
            size += temp['size']
        sys.stdout.write("Total size: %d bytes\n" % size)

def getInformation(input):
    """Find and return information about a package"""
    if len(input) != 2:
        sys.stderr.write("Invalid usage - please see help\n")
    else:
        packageName = input[1]
        try:
            pkgs = packagelist.getPackageList()
            package = pkgs.getInformation(packageName)
        except packagelist.NotFound:
            sys.stdout.write("Package not found in database\n")
            return
        information = "Name: %s\n" % package['name']
        information += "Version %s\n" % package['version']
        information += "Size: %s\n" % package['size']
        for x in package['author']:
            information += "Author: %s <%s>\n" % (x[0], x[1])
        information += "Description: %s\n" % package['description']
        sys.stdout.write(information)
        
        
def installModule(commands, help):
    """Installs commands to check dependency information and size information"""
    commands['depends'] = getDependency
    commands['available'] = listAvailable
    commands['info'] = getInformation
    help['depends'] = "depends [pkgname] to get uninstalled package dependencies and approximate size\n"
    help['available'] = "lists packages available for installation\n"
    help['info'] = "info [pkgname] returns information on [pkgname]"
