from __future__ import nested_scopes

# system modules
import os, string, sys, xml.sax

# local modules
import config, xmlparsers

__version__ = "$Revision: 1.8 $"

packageList = ''

class PackageList:
    """Holds information about available packages

    This class is used to get information about packages such
    as dependencies."""

    def __init__(self):
        """Initialize class variables

        Reads package information from a file called packages and then
        tries to parse it to extract dependency information.  This file
        probably should later try to obtain package information from
        a non-local repository at some point."""

        try:
            packageParser = xmlparsers.PackageReader()
            xml.sax.parse(os.path.expanduser('~/.ciphon/packages.xml'),
                          packageParser)
         
        except xmlparsers.InvalidXML, x:
            print "Error parsing packages.xml: ", x
            
        self.__pkgList = packageParser.getPackageList()

    def getVersion(self, packageName):
        """Return the current version for a package"""
        if self.__pkgList.has_key(packageName):
            temp = self.__pkgList[packageName]
            return temp['version']
        else:
            raise NotFound()

    def getLocation(self, packageName):
        """Return the location of a package"""
        if self.__pkgList.has_key(packageName):
            temp = self.__pkgList[packageName]
            return temp['path']
        else:
            raise NotFound()

    def getDepends(self, packageName):
        """Return dependencies for a package"""
        if self.__pkgList.has_key(packageName):
            temp = self.__pkgList[packageName]
            return temp['dependencies']
        else:
            raise NotFound()

    def getSize(self, packageName):
        """Return size of a package"""
        if self.__pkgList.has_key(packageName):
            temp = self.__pkgList[packageName]
            return temp['size']
        else:
            raise NotFound()

    def getInformation(self, packageName):
        """Return information for packageName"""
        if self.__pkgList.has_key(packageName):
            return self.__pkgList[packageName]
        else:
            raise NotFound()

        
    def listAll(self):
        """Returns a list with a name, version, size tuple for each available
        package"""

        packages = []
        for packageName in self.__pkgList.keys():
            packages.append((packageName,
                                             self.getVersion(packageName),
                                             self.getSize(packageName)))
        return packages

def getPackageList():
    """Return an instance of PackageList

    Currently this function returns a new instance of PackageList.  Later
    on, this function should try to create a single instance of PackageList
    and try to return a reference to it.  E.g. make PackageList a singleton
    class"""
    global packageList
    if packageList:
        return packageList
    else:
        packageList = PackageList()
        return packageList


# ************* Exceptions thrown by Installer class *****************

class NotFound(Exception):
    """Indicates that a package is not found in the package database"""
    pass
